#include "Camera.h"
#include <glm\gtx\vector_angle.hpp>
#include <iostream>
#include "../Settings.h"

Camera::Camera()
{
	isLeft = false;
	rotation = 360.0f + ANGLE_BORDER;
	isDebugMode = Settings::getDebugMode();
}



Camera::~Camera(void)
{
}

void Camera::update(float deltaT) 
{
	if(isDebugMode) {
		positionDebug += vectorDebug * deltaT;

	} else {
		if(isLeft) {
		
			if(rotation > 360.0f - ANGLE_BORDER) {
				rotation -= ANGLE_STEP * deltaT;
			}
			else rotation =  360.0f-ANGLE_BORDER;	
		} else {
		
			if(rotation < 360.0f + ANGLE_BORDER) {
				rotation += ANGLE_STEP * deltaT;
			}
			else rotation = 360.0f + ANGLE_BORDER;
		}
	}
}

void Camera::changeCameraAngle()
{
	std::cout << "change site isLeft: " << isLeft << std::endl;
	isLeft = !isLeft;
}

glm::mat4 Camera::getViewMatrix(glm::vec3 positionHero, glm::vec3 positionLight)
{
	glm::vec3 positionSource = glm::vec3(
		(positionLight.x+(positionHero.x-positionLight.x)*2/3),
		(positionLight.y+(positionHero.y-positionLight.y)*2/3),
		(positionLight.z+(positionHero.z-positionLight.z)*2/3)
	);

	//positionSource = positionHero;

	glm::mat4 viewMatrix;
	if(isDebugMode)	{
		viewMatrix = glm::lookAt(
			positionDebug,
			positionSource,
			glm::vec3(0.0f, 1.0f, 0.0f)
		);
	} else {
		float distance = glm::distance(positionHero, positionLight);
		glm::vec4 moveVector =glm::vec4(0.0f, distance/3.0f, 1.0f + distance*0.75,1.0f);
		glm::mat4 rotationMatrix = glm::rotate(rotation, glm::vec3(0.0f, 1.0f, 0.0f));
		glm::vec4 rotatedVec = rotationMatrix * moveVector;

		glm::mat4 resMat = glm::translate(glm::vec3(rotatedVec));
		glm::vec4 vector4 = resMat * glm::vec4(positionSource, 1.0f);
		glm::vec3 vector3 = glm::vec3(vector4.x, vector4.y, vector4.z);

		position = vector3;
		positionDebug = position;

		viewMatrix= glm::lookAt(
			position,
			positionSource,
			glm::vec3(0.0f, 1.0f, 0.0f)
		);
	}

	return viewMatrix;
}

glm::vec3 Camera::getPosition() {
	/*std::cout << position.x << " " << position.y << " " << position.z << std::endl;*/
	return (isDebugMode) ? positionDebug : position;
}

void Camera::changeDebugMode() {
	isDebugMode = !isDebugMode;
}

void Camera::setDebugVector(glm::vec3 vector)
{
	vectorDebug = vector;
}

