#include "Scene.h"
#include "../Settings.h"
#include <algorithm>

Scene::Scene(int frameWidth, int frameHeight, Window *window)
	:windowSize(frameWidth, frameHeight)
	, timer2Second(0.0f)
{
	init(frameWidth, frameHeight, window);	
}

Scene::~Scene(void)
{
}

void Scene::init(int frameWidth, int frameHeight, Window *window) 
{
	shaderManager = ShaderManager::getInstance();
	help = 0;
	isDebugMode = Settings::getDebugMode();
	camera = new Camera();
	currentLevel = 1;
	
	
	shadowFBO = new FrameBufferObject();
	shadowFBO->init((int)Settings::getWidth()*4, (int)Settings::getHeight()*4);

	glow1FBO = new FrameBufferObject();
	glow1FBO->init((int)Settings::getWidth()*0.25, (int)Settings::getHeight()*.25);

	glow2FBO = new FrameBufferObject();
	glow2FBO->init((int)Settings::getWidth()*0.25, (int)Settings::getHeight()*.25);

	glow3FBO = new FrameBufferObject();
	glow3FBO->init((int)Settings::getWidth()*0.25, (int)Settings::getHeight()*.25);

	SoundManager::init();
	std::cout << "playsound - ambient " << std::endl;
	SoundManager::playSound(SoundManager::SOUND_COLLECTION::AMBIENT_MEDIUM, SoundManager::SOUND_SOURCE::WORLD, true);


	KeyboardManager::init(window);
	ModelManager::init();

	window->setKeyboard(KeyboardManager::keyboardInput);

	projectionMatrix = glm::perspective(
		60.0f,
		((float)frameWidth) / frameHeight,
		0.01f, 
		30.0f
	);		

	Bar * _bar = new Bar(glm::vec3(-0.90, -0.96, 0.0), glm::vec3(1.0, 0.0, 0.0));
	_bar->updateParam(0.04f);
	lifesleft.push_back(new Bar(*_bar));
	_bar->setPosition(glm::vec3(-0.80, -0.96, 0.0));
	lifesleft.push_back(new Bar(*_bar));
	_bar->setPosition(glm::vec3(-0.70, -0.96, 0.0));
	lifesleft.push_back(new Bar(*_bar));
	_bar->setPosition(glm::vec3(-0.60, -0.96, 0.0));
	lifesleft.push_back(new Bar(*_bar));
	_bar->setPosition(glm::vec3(-0.50, -0.96, 0.0));
	lifesleft.push_back(new Bar(*_bar));
	_bar->setPosition(glm::vec3(-0.40, -0.96, 0.0));
	lifesleft.push_back(new Bar(*_bar));

	startNextLevel();
}

void Scene::draw(SceneObject::RENDERTYPE mode) 
{
	objectDrawn = 0;
	float distance = glm::distance(hero->getPosition(), flame->getPosition());
	//std::cout << "scalar max: " << std::min(1.0f, std::max(0.0f, (distance - maxDistance-1)/2)) <<  "distance : "<< distance << " minDistance : "<< minDistance<< " maxDistance : "<< maxDistance << std::endl;
	//glUniform1i(glGetUniformLocation(shader->programHandle, "isDebugMode"), (int)isDebugMode);
	if(mode == SceneObject::RENDERTYPE::DEPTHSHADER) {

		shadowFBO->ClearColorWhite();
		shadowFBO->BindForWriting();
		//depthShader->useShader();
		shaderManager->useShader(ShaderManager::SHADER::DEPTHSHADER);

		glm::mat4 projMat = glm::perspective(
			160.0f,
			(float)(windowSize.x) / (float)windowSize.y,
			0.01f, 
			10.0f
		);	


		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::DEPTHSHADER, "projectionMatrix"), 1, 
			GL_FALSE, glm::value_ptr(projMat));
		
		glm::vec3 lightpos = (isDebugMode) ? flame->getPositionDebug() : flame->getPosition();

		glm::mat4 viewMat = glm::lookAt(
			lightpos,
			hero->getPosition(),
			glm::vec3(0.0f, 1.0f, 0.0f)
		);

		viewMat = projMat * viewMat;
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::DEPTHSHADER, "viewMatrix"), 1, 
			GL_FALSE, glm::value_ptr(viewMat));
		
		if(isDebugMode == false) {
			glUniform3fv(
				shaderManager->getUniformLoc(ShaderManager::SHADER::DEPTHSHADER,"camPosition"), 1,
				glm::value_ptr(flame->getPosition()));
		} else {
			glUniform3fv(
				shaderManager->getUniformLoc(ShaderManager::SHADER::DEPTHSHADER,"camPosition"), 1,
				glm::value_ptr(flame->getPositionDebug()));
		}
	} else if(mode == SceneObject::RENDERTYPE::BASIC  || mode == SceneObject::RENDERTYPE::TRANSPARENCY) {
		//BasicShader
		//shader->useShader();
		shaderManager->useShader(ShaderManager::SHADER::BASICSHADER);
		
		glm::mat4 biasMat(
		0.5, 0.0, 0.0, 0.0,
		0.0, 0.5, 0.0, 0.0,
		0.0, 0.0, 0.5, 0.0,
		0.5, 0.5, 0.5, 1.0
		);
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "biasMat"), 1, 
			GL_FALSE, glm::value_ptr(biasMat));

		glm::mat4 depthProjMat = glm::perspective(
			160.0f,
			(float)(windowSize.x) / (float)windowSize.y,
			0.01f,
			10.0f
		);

		glm::vec3 lightpos = (isDebugMode) ? flame->getPositionDebug() : flame->getPosition();
		glm::mat4 depthViewMat = glm::lookAt(
			lightpos,
			hero->getPosition(),
			glm::vec3(0.0f, 1.0f, 0.0f)
		);

		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "depthProjMat"), 1, 
			GL_FALSE, glm::value_ptr(depthProjMat));
		
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "depthViewMat"), 1, 
			GL_FALSE, glm::value_ptr(depthViewMat));

		glm::mat4 depthMVP = biasMat * depthProjMat * depthViewMat;
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "depthMVP"), 1, 
			GL_FALSE, glm::value_ptr(depthMVP));

		shadowFBO->BindForReadingDepthMap(GL_TEXTURE0+2);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER,"shadow_texture"), 2);

		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "projectionMatrix"), 1, 
			GL_FALSE, glm::value_ptr(projectionMatrix));

		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "viewMatrix"), 1, 
			GL_FALSE, glm::value_ptr(camera->getViewMatrix(hero->getPosition(),flame->getPosition())));

		glUniform3fv(
				shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER,"camPosition"), 1,
				glm::value_ptr(camera->getPosition()));

		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER,"border"), (int)(windowSize.y - windowSize.y*0.1));

		//std::cout << "blackborder " <<  windowSize.y - windowSize.y*0.1 << std::endl;
	} else if(mode == SceneObject::RENDERTYPE::FLAMESHADER) {

		shaderManager->useShader(ShaderManager::SHADER::FLAMESHADER);

		glow1FBO->ClearColorTransparent();
		glow1FBO->BindForWriting();

		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::FLAMESHADER, "projectionMatrix"), 1, GL_FALSE, glm::value_ptr(projectionMatrix));
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::FLAMESHADER, "viewMatrix"), 1, GL_FALSE, glm::value_ptr(camera->getViewMatrix(hero->getPosition(), flame->getPosition())));
		
	} else if(mode==SceneObject::RENDERTYPE::BLOOMSHADER) {
		//Add GlowEffekt
		//glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"glowTexture"), 0);
		//flame->draw(mode);s

		//1st pass
		shaderManager->useShader(ShaderManager::SHADER::BLOOMSHADER);

		glow2FBO->BindForWriting();
		glow1FBO->BindForReadingTextureMap(GL_TEXTURE0);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"glowTexture"), 0);
		
		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"windowHeight"), windowSize.y*0.25);
		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"windowWidth"), windowSize.x *0.25);

		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"pass"), 1);
		plane->draw(mode);


		//2nd pass
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glow3FBO->BindForWriting();
		glow2FBO->BindForReadingTextureMap(GL_TEXTURE0);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"glowTexture"), 0);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"pass"), 2);
		plane->draw(mode);
		
		//3rd pass
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glViewport(0, 0, windowSize.x, windowSize.y);
		glow3FBO->BindForReadingTextureMap(GL_TEXTURE0);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"glowTexture"), 0);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"pass"), 3);
		glUniform1i(shaderManager->getUniformLoc(ShaderManager::SHADER::BLOOMSHADER,"border"), (int)(windowSize.y - windowSize.y*0.1));

	} else if(mode == SceneObject::RENDERTYPE::HUDSHADER) {
		//hudShader->useShader();
		shaderManager->useShader(ShaderManager::SHADER::HUDSHADER);
	}

	if(mode!= SceneObject::RENDERTYPE::TRANSPARENCY) {
		hero->draw(mode);
		objectDrawn++;

		for(std::vector<Cube *>::iterator it = world.begin(); it != world.end(); ++it){
			if(!(*it)->isDead()) {

				//viewFrustum Culling
				if(KeyboardManager::isViewFrustumEnabled() == false || ((*it)->getPosition().z < flame->getPosition().z+2 && (*it)->getPosition().z > flame->getPosition().z-8))
				{
					(*it)->draw(mode);
					objectDrawn++;
				}
			}
		}
		flame->draw(mode);
		objectDrawn++;
		propeller->draw(mode);
		objectDrawn++;
		plane->draw(mode);
		objectDrawn++;
	}
	
	if(mode == SceneObject::RENDERTYPE::TRANSPARENCY) {
		bool transparency = KeyboardManager::isInvisabilityEnabled();
		for(std::vector<CubeInvis *>::iterator it = worldInvis.begin(); it != worldInvis.end(); ++it){
			if(!(*it)->isDead()) {
				if(KeyboardManager::isViewFrustumEnabled() == false || ((*it)->getPosition().z < flame->getPosition().z+2 && (*it)->getPosition().z > flame->getPosition().z-8))
				{
					(*it)->draw(mode, transparency);
					objectDrawn++;
				}
			}
		}
	}
	barCold->draw(mode);
	barHeat->draw(mode);

	int count = 0;
	for(std::vector<Bar *>::iterator it = lifesleft.begin(); it != lifesleft.end(); ++it){
		if(count == hero->getLives()) break;
		(*it)->draw(mode);
		count++;
	}

	shadowFBO->UnBindForWriting();
}

void Scene::update(GLdouble deltaT)
{
	getKeyInput((float) deltaT);

	if(isDebugMode) {
		camera->setDebugVector(KeyboardManager::getVectorDebugCam());
		camera->update((float)deltaT);
		flame->setDebugVector(KeyboardManager::getVectorDebugLight());
		flame->update((float) deltaT);
		propeller->update(hero->getModelMatrix(),deltaT);
	} else 	{
		camera->update((float) deltaT);

		hero->update(deltaT, flame->getPosition());
		propeller->update(hero->getModelMatrix(), deltaT);

		computeGravity();
		flame->setPositionHero(hero->getPosition());
		flame->update(deltaT);

		if(cubeFinish!=nullptr) {
			cubeFinish->setPositionHero(hero->getPosition());
			cubeFinish->update((float)deltaT);

			if(cubeFinish->hasWon()) 
			{
				std::cout << "you WON" << std::endl;
				currentLevel++;

				if(currentLevel>3) {
					gameover = true;
				} else startNextLevel();
			}
		}

		for(std::vector<Cube *>::iterator it = world.begin(); it != world.end(); ++it){
			(*it)->setPositionHero(hero->getPosition());
			(*it)->update((float)deltaT);
		}

		for(std::vector<CubeInvis *>::iterator it = worldInvis.begin(); it != worldInvis.end(); ++it){
			(*it)->setPositionHero(hero->getPosition());
			(*it)->update((float)deltaT);
			if((*it)->isDead()) {
				int valueAt = ((int) (*it)->getPosition().z*-5) + ((int) (*it)->getPosition().x);
				mapHeightVec.at(valueAt) = 0;
			}
		}

		updateHud(deltaT);

		for(std::vector<Bar *>::iterator it = lifesleft.begin(); it != lifesleft.end(); ++it){
			(*it)->update(deltaT);
		}
	}

	if(hero->hasDied())
	{
		std::cout << "hero died" << std::endl;

		if(hero->getLives() < 1) {
			std::cout << "Useless Shit" << std::endl;
			gameover = true;
		} else {
			std::cout << hero->getLives() << " lives left" << std::endl;
			hero->resetNewLive();
			hero->setPosition(startingPoint + glm::vec3(0.0f, 0.1f, 0.0f));
			flame->setPosition(startingPoint + glm::vec3(0.0f, 0.05f, 5.0f));
		}
	}
}

void Scene::computeGravity(void)
{
	glm::vec3 heroPos = hero->getPosition();

	int heroHeight[3] = {(int)heroPos.x, (int)heroPos.y, (int)heroPos.z};

	//std::cout << "heroPos" << heroHeight[0] << " " << heroHeight[1] << " " << heroHeight[2] << " ResultAT: " << ((int) heroPos.z)*-5 + ((int)heroPos.x+2) << std::endl; 
	
	
	int valueAt = ((int) heroPos.z)*-5 + ((int)heroPos.x);
	if(valueAt != currentPositionInCubeWorld) {
		currentPositionInCubeWorld = valueAt;
		//std::cout << "NewValue auf: " << valueAt << " " << (int)hero->getPosition().x << " " << (int)hero->getPosition().z  << std::endl;
		//std::cout << (int) -1.5f << " " << (int) -0.5f << " " << (int) 0.5f << " " << (int) 1.5f << std::endl;
	}
	int minHeight = mapHeightVec.at(valueAt);

	hero->setGravity(minHeight);
}

void Scene::getKeyInput(float deltaT) 
{
	int key = KeyboardManager::getKey();
	
	float isInAir = (std::abs(hero->getPosition().y - mapHeightVec.at(currentPositionInCubeWorld)) < 0.1f) ? 2.5 : 1.5;
	if(!isDebugMode) hero->move(KeyboardManager::getVectorPlayer()*isInAir);

	switch(key) {
	case GLFW_KEY_ESCAPE:
		gameover=true;
	case GLFW_KEY_SPACE:
		if(!isDebugMode) hero->jump();
		break;
	case GLFW_KEY_ENTER:
		if(!isDebugMode) camera->changeCameraAngle();
		break;
	case GLFW_KEY_TAB:
		//camera->changeCameraAngle();
		isDebugMode = !isDebugMode;
		camera->changeDebugMode();
		flame->changeDebugMode();
		break;
	case GLFW_KEY_1:
		//hero->playSound(SceneObject::SOUND_COLLECTION,false);
		break;
	case GLFW_KEY_F4:
		//updateall Objects Texture Sampling
		updateAllObjectsTexture(KeyboardManager::isTextureSamplingLinear(), false);
		writeInformation(deltaT);
		break;
	case GLFW_KEY_F5:
		//updateall Objects MipMapping
		updateAllObjectsTexture(KeyboardManager::levelOfMipMapping(), true);
		writeInformation(deltaT);
		break;
	case 0: break;
	default: break;
	}
}

void Scene::clear() 
{
}

bool Scene::addCube(Cube *cube){

	world.push_back(cube);
	//std::cout << "new cube at " << cube->getPosition().x << " " << cube->getPosition().y << " " << cube->getPosition().z << std::endl;
	return true;
}

// http://runnable.com/UpSpP5ymsslIAAFo/reading-a-file-into-an-array-for-c%2B%2B-and-fstream
glm::vec3 Scene::loadLevel(std::string level, CubeFinish **cubeF)
{
	std::cout << "LADE das LEVEL: " << level << std::endl;
	std::string a = "ExFlame/Levels/" + level + ".lvl";
	glm::vec3 startPosition = glm::vec3(0.0f);

	int array_size = 1024; // define the size of character array
	char * array = new char[array_size]; // allocating an array of 1kb
	int position = 0; //this will be used incremently to fill characters in the array 
	//mapHeight = new int[array_size];
	//mapHeightVec.resize(512);
  
	std::ifstream fin(a); //opening an input stream for file test.txt
		/*checking whether file could be opened or not. If file does not exist or don't have read permissions, file
	  stream could not be opened.*/
	if(fin.is_open())
	{
		//file opened successfully so we are here
		std::cout << "File Opened successfully!!!. Reading data from file into array" << std::endl;
		int height = 0, type = 0;
		while(!fin.eof() && position < array_size)
		{
			char next = fin.get();
			if(next == '\n') next = fin.get();
			if(position%2 == 0) {
				if(next != '0')
				{
					switch(next) {
					case '1': height = 1; break;
					case '2': height = 2; break;
					case '3': height = 3; break;
					case '4': height = 4; break;
					case '5': height = 5; break;
					default: height = 0;
					}
				} else height = 0;
			} else {
				int x = position%10/2;
				int z = position/10;
				
				if(height > 0) 
				{
					if(next == 'S') {
						startPosition = glm::vec3(x+0.5f, (float) height, -z-0.5f);
					} else if(next == 'Z') {
						*cubeF = new CubeFinish(glm::vec3(x, (float) height, (-1)*z));
					}
					
					switch(next) {
					case '2': 
						{
							//CubeInvis *newCubeInvis = new CubeInvis(glm::vec3(x, (float) height, (-1)*z), shader, depthShader, flameShader);
							CubeInvis * newCubeInvis = (CubeInvis *) ModelManager::getNewObject(ModelManager::CUBE_INVIS, glm::vec3(x, (float) height, (-1)*z));
							worldInvis.push_back(newCubeInvis);
							std::cout<< newCubeInvis->getPosition().x << " " << newCubeInvis->getPosition().y << " " << newCubeInvis->getPosition().z << std::endl;
							//world.push_back(newCubeInvis);
							break;
						}
					default:
						{
							//Cube *newCube = new Cube(glm::vec3(x, (float) height, (-1)*z),shader, depthShader, flameShader);
							Cube * newCube = (Cube*) ModelManager::getNewObject(ModelManager::CUBE_STANDARD, glm::vec3(x, (float) height, (-1)*z));
							addCube(newCube);	
						}
					}					
				}
				//mapHeight[position%2] = height;
				mapHeightVec.push_back(height);
			}

			array[position] = next;
			//fin.get(array[position]); //reading one character from file to array
			position++;
		}
		array[position-1] = '\0'; //placing character array terminating character
    }
	else //file could not be opened
	{
		std::cout << "File could not be opened." << std::endl;
	}
	std::cout << "InvisCubes: " << worldInvis.size() << std::endl;
	return startPosition;
}

void Scene::startNextLevel(void)
{
	currentPositionInCubeWorld = 0;
	std::cout << "Current Level " << currentLevel << std::endl; 
	mapHeightVec.clear();
	world.clear();
	worldInvis.clear();
	cubeFinish = nullptr;
	startingPoint = loadLevel(std::to_string(currentLevel), &cubeFinish);
	
	gameover = false;
	//hero = new Hero(startingPoint,shader, depthShader, flameShader);
	hero = (Hero *)ModelManager::getNewObject(ModelManager::HERO, startingPoint + glm::vec3(0.0f, 0.05f, 0.0f));

	//propeller = new Propeller(glm::vec3(0.0f), shader, depthShader, flameShader); 
	propeller = (Propeller *) ModelManager::getNewObject(ModelManager::PROPELLER,glm::vec3(0.0f));
	//hero = (Hero *) ModelManager::getNewObject(ModelManager::MM_TYPE::HERO, startPos);
	//flame = new Flame(startingPoint + glm::vec3(0.0f, 0.05f, -3.0f), shader, depthShader, flameShader);
	flame = (Flame *) ModelManager::getNewObject(ModelManager::FLAME, startingPoint + glm::vec3(0.0f, 0.5f, 5.0f));
	flame->setPosition(startingPoint + glm::vec3(0.0f, 0.05f, 5.0f));
	//flame = (Flame *)ModelManager::getNewObject(ModelManager::MM_TYPE::FLAME, startPos+glm::vec3(0.0f, 0.5f, -2.0f));
	plane = new Plane(glm::vec3(0.0f));

	barHeat = new Bar(glm::vec3(0.0f, -0.96f , 0.0f), glm::vec3(1.0f, 0.0f, 0.0f));
	barCold = new Bar(glm::vec3(0.0, -0.9f,0.0f), glm::vec3(0.2f, 0.2f, 1.0f));
}

void Scene::loadModels(void)
{

}

bool Scene::gameIsOver(void)
{
	return gameover;
}

void Scene::updateHud(float deltaT)
{
	barHeat->updateParam(hero->getDamageHeat());
	barCold->updateParam(hero->getDamageFrozen());

	barHeat->update(deltaT);
	barCold->update(deltaT);
}

void Scene::writeInformation(float deltaT)
{
	if(KeyboardManager::isInformationRequired()){
		std::cout << "---------------------------------------------------------------------" <<std::endl;
		
		std::cout << "draw Wireframe: " << !KeyboardManager::isFillPolygonsEnabled() << std::endl; 
		std::cout << "Transparency: " << !KeyboardManager::isInvisabilityEnabled() << std::endl;
		std::cout << "ViewFrustum: " << !KeyboardManager::isViewFrustumEnabled()<< " | " << objectDrawn << " object drawn" << std::endl;
		std::cout << "TextureSampling: " << !KeyboardManager::isTextureSamplingLinear() << std::endl;
		std::cout << "Level of MipMapping: " << !KeyboardManager::levelOfMipMapping() << std::endl;
		timer2Second = 0.0f;

		if(!KeyboardManager::isFillPolygonsEnabled()) {
			glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
		} else {
			glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
		}
	} else if(timer2Second > 1.0f && KeyboardManager::isFrameTimeEnabled()) {
		std::cout << "---------------------------------------------------------------------" << std::endl;
		std::cout << "Frametime: " << deltaT << " " << (int) (1/deltaT) << "FPS" << std::endl; 
		timer2Second = 0.0f;
	}
	timer2Second += deltaT;
}          

void Scene::updateAllObjectsTexture(int mode, bool mipMapping) 
{
	if(mipMapping) {

		hero->setMipMapping(mode);
		for(std::vector<Cube *>::iterator it = world.begin(); it != world.end(); ++it){
			(*it)->setMipMapping(mode);
		}
		for(std::vector<CubeInvis *>::iterator it = worldInvis.begin(); it != worldInvis.end(); ++it){
			(*it)->setMipMapping(mode);
		}
		flame->setMipMapping(mode);
		propeller->setMipMapping(mode);
		plane->setMipMapping(mode);
	} else {
		hero->setTextureSampling(mode);
		for(std::vector<Cube *>::iterator it = world.begin(); it != world.end(); ++it){
			(*it)->setTextureSampling(mode);
		}
		for(std::vector<CubeInvis *>::iterator it = worldInvis.begin(); it != worldInvis.end(); ++it){
			(*it)->setTextureSampling(mode);
		}
		flame->setTextureSampling(mode);
		propeller->setTextureSampling(mode);
		plane->setTextureSampling(mode);
	}
}