#pragma once

#include <iostream>
#include <vector>
#include <gl\glew.h>
#include <glfw\glfw3.h>
#include <glm\glm.hpp>
#include <glm\gtc\type_ptr.hpp>
#include <glm\gtc\matrix_transform.hpp>

#include "..\Objects\SceneObject.h"
#include "..\Objects\Shader.h"
#include "Camera.h"
#include "KeyboardManager.h"
#include "../Objects/SOTypes/Hero.h"
#include "../Objects/SOTypes/Flame.h"
#include "../Objects/SOTypes/Cube.h"
#include "../Objects/SOTypes/CubeFinish.h"
#include "../Objects/SOTypes/CubeInvis.h"
#include "../Objects/SOTypes/Propeller.h"
#include "../Objects/SimpleObjects/Plane.h"
#include "../Objects/SimpleObjects/Bar.h"
#include "SoundManager.h"
#include "FrameBufferObject.h"
#include "../ModelManager.h"
#include "../GameEngine/ShaderManager.h"

#include <fstream>
#include <string>
#include <Windows.h>

class Scene
{
public:
	Scene(int frameWidth, int frameHeight, Window *window);
	~Scene(void);

	void clear();
	void draw(SceneObject::RENDERTYPE mode);
	void update(GLdouble deltaT);
	bool gameIsOver(void);
	void startNextLevel(void);
	void writeInformation(float deltaT);

private:
	void init(int frameWidth, int frameHeight, Window *window);
	void getKeyInput(float deltaT);
	glm::vec3 loadLevel(std::string level, CubeFinish **cubeF);
	bool addCube(Cube *object);
	void loadModels(void);
	void computeGravity(void);
	void updateHud(float deltaT);
	void updateAllObjectsTexture(int mode, bool MipMapping);

	int help;
	int currentPositionInCubeWorld;
	float timer2Second;
	int objectDrawn;

	std::vector<Cube *> world;
	std::vector<CubeInvis *> worldInvis;
	std::vector<Bar *> lifesleft;
	Bar *barCold, *barHeat;
	Hero *hero;
	Flame *flame;
	CubeFinish *cubeFinish;
	Plane *plane;
	Propeller *propeller;
	Camera *camera;
	FrameBufferObject *shadowFBO, *glow1FBO, *glow2FBO, *glow3FBO;

	bool isDebugMode;
	bool gameover;
	float maxDistance, minDistance;
	std::vector<int> mapHeightVec;

	glm::vec2 windowSize;
	glm::vec3 startingPoint;
	glm::mat4 projectionMatrix;

	int currentLevel;
	ShaderManager *shaderManager;

};

