#include "ShadowMapping.h"


ShadowMapping::ShadowMapping(void)
{
}


ShadowMapping::~ShadowMapping(void)
{
}

bool ShadowMapping::init(int WindowWidth, int WindowHeight)
{
	m_fbo = 0;
	// Create the FBO
    glGenFramebuffers(1, &m_fbo);

	//Create Texture
	m_renderedTexture=0;
	glGenTextures(1, &m_renderedTexture);
	glBindTexture(GL_TEXTURE_2D, m_renderedTexture);
	//Empty Image to Opengl
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, WindowWidth*5, WindowHeight*5, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);


	m_renderedTextureSmall=0;
	glGenTextures(1, &m_renderedTextureSmall);
	glBindTexture(GL_TEXTURE_2D, m_renderedTextureSmall);
	//Empty Image to Opengl
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, WindowWidth/2, WindowHeight/2, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

	m_depth = 0;
	glGenTextures(1, &m_shadowMap);
	glBindTexture(GL_TEXTURE_2D, m_shadowMap);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT24, WindowWidth*5, WindowHeight*5, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_REF_TO_TEXTURE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, GL_LESS); 

	glBindFramebuffer(GL_FRAMEBUFFER, m_fbo);
	glFramebufferTexture2D(GL_DRAW_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, m_renderedTexture, 0);
	glFramebufferTexture2D(GL_DRAW_FRAMEBUFFER, GL_COLOR_ATTACHMENT1, GL_TEXTURE_2D, m_renderedTextureSmall, 0);
	glFramebufferTexture2D(GL_DRAW_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_TEXTURE_2D, m_shadowMap, 0);

    //glFramebufferTexture2D(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_TEXTURE_2D, m_depth, 0);
	
    //Disable writes to the color buffer

    // Disable reads from the color buffer
    //glReadBuffer(GL_NONE);

	GLenum DrawBuffers[2] = {GL_COLOR_ATTACHMENT0, GL_COLOR_ATTACHMENT1};
	glDrawBuffers(2, DrawBuffers);

    GLenum Status = glCheckFramebufferStatus(GL_FRAMEBUFFER);

    if (Status != GL_FRAMEBUFFER_COMPLETE) {

		std::cout << "FB error, status: 0x" << Status << std::endl;
        return false;
    }

	glBindFramebuffer(GL_FRAMEBUFFER, 0);
    return true;
}

void ShadowMapping::BindForWriting()
{
    glBindFramebuffer(GL_DRAW_FRAMEBUFFER, m_fbo);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

void ShadowMapping::ClearColorWhite()
{
	glClearColor(1.0f, 1.f, 1.f, 1.0f);
}

void ShadowMapping::PrepareForShadowMap()
{
	glDrawBuffer(GL_NONE);
	glReadBuffer(GL_NONE);
}

void ShadowMapping::PrepareForFlameMap(int i)
{
	glDrawBuffer(GL_COLOR_ATTACHMENT0+i);
	glReadBuffer(GL_COLOR_ATTACHMENT0+i);
}

void ShadowMapping::ClearColorTransparent()
{
	glClearColor(0.0f, 0.f, 0.f, 0.0f);
}


void ShadowMapping::UnBindForWriting()
{
    glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
}


void ShadowMapping::BindForReadingTextureMap(GLenum TextureUnit)
{
    glActiveTexture(TextureUnit);
    glBindTexture(GL_TEXTURE_2D, m_renderedTexture);
}

void ShadowMapping::BindForReadingTextureMapSmall(GLenum TextureUnit)
{
    glActiveTexture(TextureUnit);
    glBindTexture(GL_TEXTURE_2D, m_renderedTextureSmall);
}

void ShadowMapping::BindForReadingDepthMap(GLenum TextureUnit)
{
    glActiveTexture(TextureUnit);
	glBindTexture(GL_TEXTURE_2D, m_shadowMap);
}