#include "Window.h"

#include <iostream>

Window::Window(int width, int height, int framerate, bool fullscreen)
	: lastTime(0)
	, width(width)
	, height(height)
	, deltaT(0)
{
	if(!glfwInit()){
		std::cout << "Failed to init glfw" << std::endl;
		system("PAUSE");
		exit(-1);
	}
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	if(framerate!=0){
		glfwWindowHint(GLFW_REFRESH_RATE, framerate);
	}

	if(fullscreen){
		window = glfwCreateWindow(width, height, "Exflame", glfwGetPrimaryMonitor(), NULL);
	}else{
		window = glfwCreateWindow(width, height, "Exflame", NULL, NULL);
	}
	
	if(!window){
		glfwTerminate();
		std::cout << "Failed to open window" << std::endl;
		system("PAUSE");
		exit(-1);
	}

	glfwMakeContextCurrent(window);
}

void Window::init(){
	glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_DISABLED);
}


Window::~Window()
{
	glfwTerminate();
}

bool Window::shouldClose(){
	return (glfwWindowShouldClose(window)==1) ? true : false;
}

void Window::swapBuffers(){

	//compute deltaT
	GLdouble time = glfwGetTime();
	deltaT = time - lastTime;
	lastTime = time;

	glfwSwapBuffers(window);
}

void Window::makeContext(){
	glfwMakeContextCurrent(window);
}

GLint Window::getFramebufferWidth(){
	GLint width;
	glfwGetFramebufferSize(window, &width, NULL);
	return width;
}

GLint Window::getFramebufferHeight(){
	GLint height;
	glfwGetFramebufferSize(window, NULL, &height);
	return height;
}

void Window::cleanUp(){
	glfwTerminate();
}

bool Window::getKey(GLint key){
	return (glfwGetKey(window, key)) ? true : false;
}

GLFWwindow * Window::getWindow(){
	return window;
}

GLdouble Window::getDeltaT(){
	return deltaT;
}

void Window::setKeyboard(GLFWkeyfun function){
	glfwSetKeyCallback(window, function);
}

void Window::resizeWindow(void)
{
	glfwSetWindowSize(window, width, height);
}
