#include "ParticleSystem.h"

ParticleSystem::ParticleSystem(glm::vec3 _position, Shader* _shader, Shader *_depthShader, Shader *_flameShader, Shader *_particleShader)
	:SceneObject("Plane/plane", _position, _shader, _depthShader, _flameShader, 1.0f, 1.0f)
	{
	NumOfInstances = 2000;
	particleShader = _particleShader;
	//loadOBJ(pathToObj, indices, vertices, uvs, normals);
	//tex->Bind(0);
	//glUniform1i(glGetUniformLocation(_shader->programHandle, "colorTexture"), 0);

	modelMats.reserve(NumOfInstances);
	for(int i =0; i<NumOfInstances/4;i++){
		modelMats.push_back(glm::translate(glm::fmat4x4(1.0f), glm::vec3(-2.0f, -2.0f,-5.0f-2.0f*i)));
		modelMats.push_back(glm::translate(glm::fmat4x4(1.0f), glm::vec3(2.0f, -2.0f,-5.0f-2.0f*i)));
		modelMats.push_back(glm::translate(glm::fmat4x4(1.0f), glm::vec3(-2.0f, 2.0f,-5.0f-2.0f*i)));
		modelMats.push_back(glm::translate(glm::fmat4x4(1.0f), glm::vec3(2.0f, 2.0f,-5.0f-2.0f*i)));
	}

	//Load data to buffer
	glGenBuffers(1, &modelMatsBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, modelMatsBuffer);
	glBufferData(GL_ARRAY_BUFFER, NumOfInstances * sizeof(glm::fmat4x4),
		&modelMats[0], GL_STREAM_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

	glBindBuffer(GL_ARRAY_BUFFER, modelMatsBuffer);
	GLint modelMatIndex = glGetAttribLocation(shader->programHandle, "modelMat");
	for(unsigned int i = 0; i < 4 ; i++) {
        glEnableVertexAttribArray(modelMatIndex + i);
        glVertexAttribPointer(modelMatIndex + i, 4, GL_FLOAT, GL_FALSE, sizeof(glm::fmat4x4), 
                                (const GLvoid*)(sizeof(GLfloat) * i * 4));
        glVertexAttribDivisor(modelMatIndex + i, 1);
    };

	GLint posParVec3Index = glGetAttribLocation(shader->programHandle, "modelMat");
	for(unsigned int i = 0; i < 4 ; i++) {
        glEnableVertexAttribArray(modelMatIndex + i);
        glVertexAttribPointer(modelMatIndex + i, 4, GL_FLOAT, GL_FALSE, sizeof(glm::fmat4x4), 
                                (const GLvoid*)(sizeof(GLfloat) * i * 4));
        glVertexAttribDivisor(modelMatIndex + i, 1);
    };

	glBindVertexArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

ParticleSystem::~ParticleSystem(){
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &indexBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteBuffers(1, &modelMatsBuffer);
	glDeleteVertexArrays(1, &vao);
}

void ParticleSystem::update(double deltaT){
	
}

void ParticleSystem::draw(RENDERTYPE mode){

	if(mode == RENDERTYPE::PARTICLESHADER) {
		SceneObject::draw(mode);
	}
}

//NEEDS TO BE FIXED FOR INSTANCING (OR NOT :D)
void ParticleSystem::updateObject(const char* pathToObj, Shader* _shader, Texture* _tex){
	//this->tex = _tex;
	//this->shader = _shader;
	//loadOBJ(pathToObj, indices, vertices, uvs, normals);
	//tex->Bind(0);
	//glUniform1i(1, 0);

	////Load data to buffer
	//glGenBuffers(1, &positionBuffer);
	//glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	//glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(float),
	//	&vertices[0], GL_STATIC_DRAW);
	//glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

	//	//Load data to buffer
	//glGenBuffers(1, &normalBuffer);
	//glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
	//glBufferData(GL_ARRAY_BUFFER, normals.size() * sizeof(float),
	//	&normals[0], GL_STATIC_DRAW);
	//glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

	////Load data to buffer
	//glGenBuffers(1, &uvBuffer);
	//glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
	//glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(float),
	//	&uvs[0], GL_STATIC_DRAW);
	//glBindBuffer(GL_ARRAY_BUFFER, 0); //unbind (mit 0)

	////Load data to buffer
	//glGenBuffers(1, &indexBuffer);
	//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
	//glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(unsigned int),
	//	&indices[0], GL_STATIC_DRAW);
	//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0); // unbind (mit 0)

	////Generate VAO
	//glGenVertexArrays(1, &vao);
	//glBindVertexArray(vao);

	//glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	//GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
	//glEnableVertexAttribArray(positionIndex);
	//glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	//glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
	//GLint normalIndex = glGetAttribLocation(shader->programHandle, "normal");
	//glEnableVertexAttribArray(normalIndex);
	//glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	//glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
	//GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
	//glEnableVertexAttribArray(uvIndex);
	//glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);
	//
	//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);

	//glBindVertexArray(0);
	//glBindBuffer(GL_ARRAY_BUFFER, 0);
	//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

void ParticleSystem::loadOBJ(const char *path, std::vector<unsigned int> &outIndices, std::vector<float> &outVertices,
							std::vector<float> &outUVs, std::vector<float> &outNormals){
	//							
	//Assimp::Importer importer;

	//const aiScene* scene = importer.ReadFile(path,
	//							aiProcess_JoinIdenticalVertices|
	//							aiProcess_Triangulate|
	//							aiProcess_OptimizeMeshes| 
	//							aiProcess_ConvertToLeftHanded
	//							);

	////std::cout << importer.GetErrorString() << std::endl;

	//aiMesh* mesh = scene->mMeshes[0];

	//int numOfFaces = mesh->mNumFaces;
	//int numOfIndices = numOfFaces * 3;

	//outIndices.resize(numOfIndices);
	//
	//for (unsigned int i=0; i < mesh->mNumFaces; ++i)
	//{
	//	const aiFace &face = mesh->mFaces[i];
	//	assert(face.mNumIndices == 3);
	//	outIndices[i*3 + 0] = face.mIndices[0];
	//	outIndices[i*3 + 1] = face.mIndices[1];
	//	outIndices[i*3 + 2] = face.mIndices[2];
	//}

	//int numOfVertices = mesh->mNumVertices;
	//outVertices.resize(numOfVertices * 3);
	//outNormals.resize(numOfVertices * 3);
	//outUVs.resize(numOfVertices * 2);
	//
	//for (unsigned int i=0; i < mesh->mNumVertices; ++i)
	//{
	//	if(mesh->HasPositions())
	//	{
	//		outVertices[i*3 + 0] = mesh->mVertices[i].x;
	//		outVertices[i*3 + 1] = mesh->mVertices[i].y;
	//		outVertices[i*3 + 2] = mesh->mVertices[i].z;
	//	}

	//	if(mesh->HasNormals())
	//	{
	//		outNormals[i*3 + 0] = mesh->mNormals[i].x;
	//		outNormals[i*3 + 1] = mesh->mNormals[i].y;
	//		outNormals[i*3 + 2] = mesh->mNormals[i].z;
	//	}

	//	if(mesh->HasTextureCoords(0))
	//	{
	//		outUVs[i*2 + 0] = (mesh->mTextureCoords[0][i].x);
	//		outUVs[i*2 + 1] = 1-(mesh->mTextureCoords[0][i].y);
	//	}
	//}

	//for(int i=0; i<outVertices.size();i++){
	//	float temp;
	//	temp=outVertices[i]*0.2f;
	//	outVertices[i] = temp;
	//}
}