#include "Flame.h"

#include <glm\gtx\vector_angle.hpp>
#include "../../Settings.h"
#include <algorithm>

Flame::Flame(glm::vec3 _position)
	: SceneObject("flame/flame", _position, 1.0f, 0.25f)
{
	isDebugMode = Settings::getDebugMode();
	positionDebug = glm::vec3(0.0f);
	material = new Material(0.0f, 1.0f, 0.1f);
	color = glm::vec3(1.0f);
	maxDistance = 4.0f;
	minDistance = 2.0f;
	
	SoundManager::setSoundVolume(0.3f);
	playSound(SoundManager::SOUND_COLLECTION::ENEMY_BURNING1,true);
}

Flame::~Flame(void)
{
}

void Flame::draw(RENDERTYPE mode)
{
	//std::cout << "light color: " << color.x << " " << color.y << " " << color.z << std::endl;
	if(mode == RENDERTYPE::DEPTHSHADER) {
		//DepthShader
		
		if(isDebugMode)
		{
			glUniform3fv(
				shaderManager->getUniformLoc(ShaderManager::SHADER::DEPTHSHADER, "light"), 1,
				glm::value_ptr(positionDebug));
		} else {
			glUniform3fv(
				shaderManager->getUniformLoc(ShaderManager::SHADER::DEPTHSHADER, "light"), 1,
				glm::value_ptr(position));
		}
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::DEPTHSHADER,  "modelMatrix"), 1, 
			GL_FALSE, glm::value_ptr(modelMatrix));
	} else if(mode == RENDERTYPE::BASIC) {
		//BasicShader	
		//maxDistance = glm::distance(position, positionHero) + 2;

		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "maxDistance"), 7);
		glUniform1f(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "minDistance"), minDistance);
		if(isDebugMode)
		{
			glUniform3fv(
				shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "light"), 1,
				glm::value_ptr(positionDebug));
		} else {
			glUniform3fv(
				shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "light"), 1,
				glm::value_ptr(position));
		}
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "modelMatrix"), 1, 
			GL_FALSE, glm::value_ptr(modelMatrix));
		glUniform3fv(
			shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "lightColor"), 1,
			glm::value_ptr(color));
		//SceneObject::draw(mode);
	} else if(mode == RENDERTYPE::FLAMESHADER) {
		//Flamemode
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::FLAMESHADER, "modelMatrix"), 1, 
			GL_FALSE, glm::value_ptr(modelMatrix));
		glUniform3fv(
			shaderManager->getUniformLoc(ShaderManager::SHADER::FLAMESHADER, "lightColor"), 1,
			glm::value_ptr(color));

		SceneObject::draw(mode);
	} else if(mode == RENDERTYPE::BLOOMSHADER) {
		//Flamemode
	} else if(mode == RENDERTYPE::TRANSPARENCY) {
	}
}

void Flame::update(GLdouble deltaT)
{
	if(isDebugMode)
	{
		positionDebug += vectorDebug * (float)deltaT;

		modelMatrix = glm::translate(positionDebug);
		//color = glm::vec3(1.0f);
	}
	else 
	{
		glm::vec3 direction = ((positionHero+glm::vec3(0.0f, 1.0f, 0.0f)) - position);
		float distance = glm::distance(positionHero.z, position.z);

		if(distance > 0.8f) 
		{
			float scalar = ((distance/5.0f) > 1.0f) ? distance/5.0f : 1.0f ;
			direction = direction / distance* scalar * 0.9f * WALKING_SPEED;// * scalar;
		} else {
			direction = glm::vec3(0.0f);
		}
		//std::cout << "DISTANCE: " << distance << ">" <<  (maxDistance-1) << std::endl;
		//Change Color of Flame
		if(distance > maxDistance - 0.5f)
		{
			//zu weit weg
			float scalar = std::min(1.0f, std::max(0.0f, (distance- maxDistance + 0.5f)));
			color = glm::vec3(1-scalar, 1-(scalar), 1.0f);
			

		} else if(distance < minDistance+0.5f) {
			//zu nah
			float scalar = std::min(1.0f, std::max(0.0f, (minDistance - distance + 0.5f) ));
			color = glm::vec3(1.0, (1-scalar*0.75f), 1-scalar);
		} else {
			color = glm::vec3(1.0); 
		}

		//Play sound and set audiovolume
		float soundVolume = 0.7f + (1-distance/2/maxDistance)*0.33f;
		SoundManager::setSoundVolume(soundVolume);

		glm::vec3 moveVec = direction * (float) deltaT;
		//position = glm::vec3(moveVec.x, positionHero.y+0.1f, moveVec.z);
		//position = glm::vec3(moveVec.x, positionHero.y+1.0f, moveVec.z);

		position += moveVec;

		glm::mat4 scalingMatrix = glm::scale(glm::vec3(scaleObject));
		glm::mat4 rotationMatrix = glm::mat4(1.0f);
		glm::mat4 transMatrix = glm::translate(moveVec) * glm::translate(position);

		modelMatrix = transMatrix;	//* scalingMatrix;// * rotationMatrix * scalingMatrix;
		//std::cout <<"Position:      " << position.x << " " << position.y << " " << position.z << std::endl;
		positionDebug = position;
		//std::cout <<"PositionDebug: " << positionDebug.x << " " << positionDebug.y << " " << positionDebug.z << std::endl;
	}	
}

void Flame::setPositionHero(glm::vec3 _position)
{
	positionHero = _position;
}

void Flame::setDebugVector(glm::vec3 vector)
{
	vectorDebug = vector;
}

void Flame::changeDebugMode()
{
	isDebugMode = !isDebugMode;
}

void Flame::playSound(SoundManager::SOUND_COLLECTION entry, bool loop)
{
	SceneObject::playSound(entry, loop);
}

glm::vec3 Flame::getPositionDebug(void) {
	return positionDebug;
}