#include "Hero.h"

Hero::Hero(glm::vec3 _position)
	: SceneObject("hero/hero", _position, 4.0f, 0.25f)
{	
	material = new Material(0.2f, 1.0f, 0.1f); 
	jumpIsPossible = false;
	jumpVec = glm::vec3(0.0f);
	health = 1.0f;
	lives = 4;
	isDead = false;
	heaten = 0.0f;
	frozen = 0.0f;
}

Hero::~Hero(void)
{
}

void Hero::draw(RENDERTYPE mode)
{
	if(mode == RENDERTYPE::DEPTHSHADER) {
		//DepthShader
		GLenum error = glGetError();
		if(error != GL_NO_ERROR) std::cout << "before sending modelMatrix - DepthShader" << std::endl;
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::DEPTHSHADER, "modelMatrix"), 1, 
			GL_FALSE, glm::value_ptr(modelMatrix));
		error = glGetError();
		if(error != GL_NO_ERROR) std::cout << "before drawing In SceneObject - DepthShader" << std::endl;
		SceneObject::draw(mode);
		error = glGetError();
		if(error != GL_NO_ERROR) std::cout << "after drawing In SceneObject - DEPTH" << std::endl;
	} else if(mode == RENDERTYPE::BASIC) {
		//BasicShader
		glUniformMatrix4fv(shaderManager->getUniformLoc(ShaderManager::SHADER::BASICSHADER, "modelMatrix"), 1, 
			GL_FALSE, glm::value_ptr(modelMatrix));
		GLenum error = glGetError();
		if(error != GL_NO_ERROR) std::cout << "before drawing In SceneObject - BASIC" << std::endl;
		SceneObject::draw(mode);
		error = glGetError();
		if(error != GL_NO_ERROR) std::cout << "after drawing In SceneObject - BASIC" << std::endl;
	} else if(mode == RENDERTYPE::FLAMESHADER) {
	} else if(mode == RENDERTYPE::TRANSPARENCY) {
	}
}

void Hero::update(GLdouble deltaT, glm::vec3 positionFlame)
{
	if(position.y>=cubeHeight) wasOnTop = true;
	//movement with keyboard
	glm::vec3 newPosition = position + direction * (float)deltaT * 0.75f;

	if(position != newPosition)
	{
		playSoundGoing();
	}

	//Gravity
	newPosition += glm::vec3(0.0f, -2.0f, 0.0f) * (float) deltaT;

	//jumpMovement
	newPosition += jumpVec * (float) deltaT;

	jumpVec += glm::vec3(0.0f, -3.0f, 0.0f) * (float) deltaT;
	if(jumpVec.y < 0.0f) jumpVec.y = 0.0f;

	//CollisionDetection
	if(((int)newPosition.y) < cubeHeight && wasOnTop==true) 
	{ 
		newPosition.y = cubeHeight;
		jumpIsPossible = true;
	} else jumpIsPossible = false;

	wasOnTop = false;

	glm::mat4 scalingMatrix = glm::scale(glm::vec3(scaleObject));
	//glm::mat4 rotationMatrix = glm::rotate(fmod(orientation.y, 1.0f) * 360.0f, glm::vec3(1.0f, 0.0f, 0.0f));
	glm::mat4 transMatrix = glm::translate(newPosition);	
	
	modelMatrix = transMatrix;; //* scalingMatrix;

	orientation.y += TURN_RATE / 360.0f * (float)deltaT;
	
	direction = glm::vec3(0.0f);
	position = newPosition;


	float distance = abs(position.z - positionFlame.z);

	if(distance < 2) {
		//FrostingDamage
		heaten += max(0.0, min(2.0, 2.0 -distance)) * deltaT * 0.4; 
		SoundManager::playSound(SoundManager::SOUND_COLLECTION::HERO_DYING_FLAME, SoundManager::SOUND_SOURCE::HERO, false);
	} else if(distance >4) {
		//HeatDamage
		frozen += max(0.0, min(3.0, distance - 4.0)) * deltaT * 0.7; 
	} else {
		frozen -= 0.05 * deltaT;
		frozen = (frozen < 0.0) ? 0.0 : frozen;
		heaten -= 0.05 * deltaT;
		heaten = (heaten < 0.0) ? 0.0 : heaten;
	}

	if( !isDead && (heaten >= 1.0 || frozen >= 1.0)) { isDead = true; }
	if(newPosition.y < 0.0) {
		isDead = true;
	}

	if(newPosition.y < 1.0) playSoundFalling();

	if(isDead) lives--;
}

bool Hero::hasDied(void)
{
	return isDead;
}

int Hero::getLives(void)
{
	return lives;
}

void Hero::resetNewLive(void)
{
	isDead = false;
	frozen = 0.0f;
	heaten = 0.0f;
}

void Hero::setGravity(int _cubeHeight)
{
	cubeHeight = _cubeHeight;
}

void Hero::move(glm::vec3 _direction)
{
	 //direction = _direction;
	direction = _direction;
	//if(direction.x != 0.0f || direction.z != 0.0f) playSound(SceneObject::SOUND_COLLECTION::HERO_JUMP, false);
}

void Hero::jump()
{
	//std::cout << "jump in hero" << std::endl;
	if(jumpIsPossible)
	{
		jumpVec = glm::vec3(0.0f, 4.8f, 0.0);
	}
	playSound(SoundManager::SOUND_COLLECTION::HERO_JUMP, false);
}

void Hero::rotate(glm::vec3 _rotation, float degree)
{
	std::cout << _rotation.x << " " << _rotation.y << " " << _rotation.z << " " << degree << std::endl;
}
void Hero::setPosition(glm::vec3 newPosition) {
	position = newPosition;
}

void Hero::die(bool wasFlame)
{
	playSoundDying();
}

void Hero::playSoundDying()
{
	//std::cout << "Sound falling hero " << std::endl;
	playSound(SoundManager::SOUND_COLLECTION::HERO_DYING_FLAME,false);
}
void Hero::playSoundGoing()
{
	playSound(SoundManager::SOUND_COLLECTION::HERO_GOING, false);
}
void Hero::playSoundJumping()
{
	playSound(SoundManager::SOUND_COLLECTION::HERO_JUMP,false);
}
void Hero::playSoundFalling()
{
	//std::cout << "hero falling to death" << std::endl;
	playSound(SoundManager::SOUND_COLLECTION::HERO_DYING_FALLING,false);
}
void Hero::playSound(SoundManager::SOUND_COLLECTION entry, bool loop)
{
	SceneObject::playSound(entry, loop);
}

bool Hero::isStanding(void)
{
	return !jumpIsPossible;
}

glm::mat4 Hero::getModelMatrix(void)
{
	return modelMatrix;
}

float Hero::getDamageFrozen(void) 
{
	return frozen;
}

float Hero::getDamageHeat(void) 
{
	return heaten;
}