#pragma once

#include <gl/glew.h>
#include <glfw/glfw3.h>
#include <glm/glm.hpp>
#include <iostream>
#include <glm\gtx\vector_angle.hpp>
#include <glm/gtc/matrix_transform.hpp>

#include "../SceneObject.h"
#include "../Shader.h"
#include "../../GameEngine/SoundManager.h"

#define TURN_RATE 15.0f

class Hero :
	public SceneObject
{
public:
	Hero(glm::vec3 _position);
	virtual ~Hero(void);

	void virtual update(GLdouble deltaT, glm::vec3 positionFlame);
	void virtual draw(RENDERTYPE mode);
	void jump();
	void move(glm::vec3 _direction);
	void rotate(glm::vec3 _rotation, float degree);
	void die(bool wasFlame);
	void playSound(SoundManager::SOUND_COLLECTION entry, bool loop) override;
	void playSoundDying();
	void playSoundGoing();
	void playSoundFalling();
	void playSoundJumping();
	void setPosition(glm::vec3 newPosition);
	void setGravity(int cubeHeight);
	int getLives(void);
	bool hasDied(void);
	void resetNewLive(void);
	bool isStanding(void);
	float getDamageFrozen(void);
	float getDamageHeat(void);

	glm::mat4 getModelMatrix(void);
private:
	int cubeHeight;
	glm::vec3 jumpVec;
	bool jumpIsPossible;
	float health;
	int lives;
	bool isDead;
	bool wasOnTop;
	float frozen, heaten;
};

