#include "Texture.h"

#include <IL\il.h>
#include <tchar.h>

Texture::Texture(const std::string& path)
{
	loadTexture(path);
	//loadTextureWithFreeImage(path);
}


Texture::~Texture(void)
{
	glDeleteTextures(1, &handle);
}

void Texture::bind(int unit)
{
	glActiveTexture(GL_TEXTURE0 + unit);
	glBindTexture(GL_TEXTURE_2D, handle);
}

void Texture::loadTexture(std::string path)
{
	ilInit();

	//fixes upside down position
	//ilOriginFunc(IL_ORIGIN_LOWER_LEFT);
	//ilEnable(IL_ORIGIN_SET);
	
	//TCHAR test = _T(path);
	
	wchar_t *wmsg = new wchar_t[strlen(path.c_str())+1]; //memory allocation
	mbstowcs(wmsg, path.c_str(), strlen(path.c_str())+1);
	
	ILuint image;

	ilGenImages(1, &image);

	ilBindImage(image);

	ilLoadImage(wmsg);

	ILenum Error;

	if((Error = ilGetError()) != IL_NO_ERROR){
		std::cout << "couldnt load texture: "  << path << "\n" << Error << std::endl;
		
		ilBindImage(NULL);
		ilDeleteImage(image);
		return;
	}

	if(!ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE)){
		std::cout << "couldnt convert texture: " << path << std::endl;
		ilBindImage(NULL);
		ilDeleteImage(image);
		return;
	}
	//std::cout << "Done: " << path << std::endl;

	data = ilGetData();
	width = ilGetInteger(IL_IMAGE_WIDTH);
	height = ilGetInteger(IL_IMAGE_HEIGHT);

	setTexture();

	ilBindImage(0);
	ilDeleteImage(image);
}

void Texture::setTexture(void)
{
	glGenTextures(1, &handle);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, handle);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data);

	glGenerateMipmap(GL_TEXTURE_2D);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glBindTexture(GL_TEXTURE_2D, 0);
}

void Texture::setTextureSampling(int mode)
{
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, handle);

	setTextureConfiguration(mode, mipMappingMode);
	textureSamplingMode = mode;

	glBindTexture(GL_TEXTURE_2D, 0);
}

void Texture::setMipMapping(int mode)
{
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, handle);
	GLenum error = glGetError();

	setTextureConfiguration(textureSamplingMode, mode);
	mipMappingMode = mode;

	glBindTexture(GL_TEXTURE_2D, 0);
}

void Texture::setTextureConfiguration(int modeTS, int modeMM)
{ 
	if(modeTS == 0) {
		//nearest neighbor
		switch(modeMM) {
		case 0: 
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
			break;
		case 1: 
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
			break;
		default:
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		}
	} else {
		//Linear
		switch(modeMM) {
		case 0: 
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			break;
		case 1:
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			break;
		default:		 
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		}
	}
}