#version 330 core

vec3 calcBumpedNormal(void);

layout(location = 0) out vec4 fragcolor; 

in vec3 fposition;
in vec3 fnormal;
in vec3 ftangents;
in vec3 fbitangents;
in vec3 toCamera;
in vec3 light;
in vec3 camPos;
in vec3 lightPosition;
in vec2 texCoord;
in mat3 TBN;
in vec4 shadowCoord;

uniform float maxDistance;
uniform float minDistance;
uniform float distance;
uniform sampler2D color_texture;
uniform sampler2D normal_texture;
uniform sampler2DShadow shadow_texture;

uniform float material_ambient;
uniform float material_diffuse;
uniform float material_specular;
uniform float transparency;
uniform int border;

uniform vec3 lightColor;
void main()
{

	float shininess = 3.0;

	vec3 H = normalize(vec3(lightPosition-fposition));
	float cosHL = max(0.05, dot(fnormal,lightPosition));

	float specular = max(0.0, dot(fnormal, H));
	specular = pow(specular, shininess);

	vec3 dis = lightPosition - fposition;
	
	//vec3 newNormal = normalize(fnormal);
	vec3 newNormal = calcBumpedNormal();
	//vec3 newNormal = normalize(TBN * (texture2D(normal_texture, texCoord).rgb*2.0 - vec3(1.0, 1.0, 1.0)));
	//newNormal = fnormal;
	float distance2 = length(dis);
	
	//SHADOW MAP CONTROL
	float visability = 1.0;
	float bias = 0.0005;
	
	vec2 coord = shadowCoord.xy/shadowCoord.w;
	float depth_r = (shadowCoord.z-bias)/shadowCoord.w;
	
	float shad = textureProj(shadow_texture, shadowCoord) * 0.5 + 0.5;

	fragcolor = shad *  (material_ambient + max(0.0, dot(newNormal, H)) * material_diffuse + pow(max(0.0,dot(newNormal, normalize(H+toCamera))), shininess)) * vec4(lightColor, 1.0) * vec4(texture(color_texture, texCoord).xyz, 1.0);
	
	if(distance2 > maxDistance-2){
		float value = min(1.0,max(0.0, (maxDistance -1 - distance2)));
		fragcolor *= vec4(vec3(value),1.0);
	}
	
	fragcolor.a = transparency;	 

	if(gl_FragCoord.y > border || gl_FragCoord.y < border/9) 
	{ 
		fragcolor = vec4(vec3(0.0),1.0);
	}
}

vec3 calcBumpedNormal()
{
	vec3 Normal = normalize(fnormal);
    vec3 Tangent = normalize(ftangents);
    Tangent = normalize(Tangent - dot(Tangent, Normal) * Normal);
//    vec3 Bitangent = cross(Tangent, Normal);
	vec3 Bitangent = normalize(fbitangents);
    vec3 BumpMapNormal = texture(normal_texture, texCoord).xyz;
    BumpMapNormal = 2.0 * BumpMapNormal - vec3(1.0, 1.0, 1.0);
    vec3 NewNormal;
    mat3 TBN = mat3(Tangent, Bitangent, Normal);
    NewNormal = TBN * BumpMapNormal;
    NewNormal = normalize(NewNormal);
    return NewNormal;
}