#version 330 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in vec3 tangents;
layout(location = 4) in vec3 bitangents;

uniform mat4 modelMatrix;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform vec3 camPosition;
uniform vec3 light;
uniform mat4 depthProjMat;
uniform mat4 depthViewMat;
uniform mat4 biasMat;
uniform mat4 depthMVP;

//Debug
//uniform vec3 lightDebug;

out vec3 fposition;
out vec3 fnormal;
out vec3 ftangents;
out vec3 fbitangents;
out vec3 toCamera;
out vec3 camPos;
out vec3 lightPosition;
out vec2 texCoord;
out mat3 TBN;
out vec4 shadowCoord;

//Debug
//out vec3 lightPositionDebug;

void main()
{
	mat3 normalMatrix = transpose(mat3(inverse(modelMatrix)));
	fposition = (modelMatrix * vec4(position,1)).xyz;
	fnormal = normalize(vec3(normalMatrix * normal));
	toCamera = normalize(vec3(camPosition - fposition));
	texCoord = uv;

	ftangents = (viewMatrix * modelMatrix * vec4(normalize(tangents),1.0)).xyz;
	fbitangents = (viewMatrix * modelMatrix * vec4(normalize(bitangents),1.0)).xyz;
	TBN = mat3(tangents, bitangents, fnormal);
	
	//fnormal = normalMatrix * normalize(normal);
	
	camPos = camPosition;
	lightPosition = light;

	gl_Position =projectionMatrix * viewMatrix * modelMatrix * vec4(position,1);

	shadowCoord = depthMVP * modelMatrix * vec4(position,1);
}